@@ -78,6 +78,10 @@ then
     export VIRTUALENVWRAPPER_PROJECT_FILENAME=".project"
 fi
 
+# Let the user tell us they never want to cd to projects
+# automatically.
+export VIRTUALENVWRAPPER_WORKON_CD=${VIRTUALENVWRAPPER_WORKON_CD:-1}
+
 # Remember where we are running from.
 if [ -z "$VIRTUALENVWRAPPER_SCRIPT" ]
 then
@@ -200,6 +204,7 @@ function virtualenvwrapper_tempfile {
     typeset file
 
     file="$(virtualenvwrapper_mktemp -t virtualenvwrapper-$suffix-XXXXXXXXXX)"
+    touch "$file"
     if [ $? -ne 0 ] || [ -z "$file" ] || [ ! -f "$file" ]
     then
         echo "ERROR: virtualenvwrapper could not create a temporary file name." 1>&2
@@ -439,8 +444,7 @@ function mkvirtualenv {
                 else
                     i=$(( $i + 1 ))
                     interpreter="${in_args[$i]}"
-                fi;
-                interpreter="$(virtualenvwrapper_absolutepath "$interpreter")";;
+                fi;;
             -r)
                 i=$(( $i + 1 ));
                 requirements="${in_args[$i]}";
@@ -517,7 +521,8 @@ function rmvirtualenv {
 
     # support to remove several environments
     typeset env_name
-    for env_name in $@
+    # Must quote the parameters, as environments could have spaces in their names
+    for env_name in "$@"
     do
         echo "Removing $env_name..."
         typeset env_dir="$WORKON_HOME/$env_name"
@@ -555,18 +560,25 @@ function virtualenvwrapper_show_workon_options {
     # NOTE: DO NOT use ls or cd here because colorized versions spew control 
     #       characters into the output list.
     # echo seems a little faster than find, even with -depth 3.
+    # Note that this is a little tricky, as there may be spaces in the path.
     #
     # 1. Look for environments by finding the activate scripts.
     #    Use a subshell so we can suppress the message printed
     #    by zsh if the glob pattern fails to match any files.
-    # 2. Strip the WORKON_HOME prefix from each name.
-    # 3. Strip the bindir/activate script suffix.
-    # 4. Format the output to show one name on a line.
+    #    This yields a single, space-separated line containing all matches.
+    # 2. Replace the trailing newline with a space, so every
+    #    possible env has a space following it.
+    # 3. Strip the bindir/activate script suffix, replacing it with
+    #    a slash, as that is an illegal character in a directory name.
+    #    This yields a slash-separated list of possible env names.
+    # 4. Replace each slash with a newline to show the output one name per line.
     # 5. Eliminate any lines with * on them because that means there 
     #    were no envs.
     (virtualenvwrapper_cd "$WORKON_HOME" && echo */$VIRTUALENVWRAPPER_ENV_BIN_DIR/activate) 2>/dev/null \
-        | command \sed "s|/$VIRTUALENVWRAPPER_ENV_BIN_DIR/activate||g" \
-        | command \fmt -w 1 \
+        | command \tr "\n" " " \
+        | command \sed "s|/$VIRTUALENVWRAPPER_ENV_BIN_DIR/activate |/|g" \
+        | command \tr "/" "\n" \
+        | command \sed "/^\s*$/d" \
         | (unset GREP_OPTIONS; command \egrep -v '^\*$') 2>/dev/null
 }
 
@@ -659,10 +671,23 @@ function virtualenvwrapper_workon_help {
     echo ""
     echo "           Show this help message."
     echo ""
+    echo "       workon (-c|--cd) envname"
+    echo ""
+    echo "           After activating the environment, cd to the associated"
+    echo "           project directory if it is set."
+    echo ""
+    echo "       workon (-n|--no-cd) envname"
+    echo ""
+    echo "           After activating the environment, do not cd to the"
+    echo "           associated project directory."
+    echo ""
 }
 
 #:help:workon: list or change working virtualenvs
 function workon {
+    typeset -a in_args
+    typeset -a out_args
+
     in_args=( "$@" )
 
     if [ -n "$ZSH_VERSION" ]
@@ -673,6 +698,7 @@ function workon {
         i=0
         tst="-lt"
     fi
+    typeset cd_after_activate=$VIRTUALENVWRAPPER_WORKON_CD
     while [ $i $tst $# ]
     do
         a="${in_args[$i]}"
@@ -680,19 +706,39 @@ function workon {
             -h|--help)
                 virtualenvwrapper_workon_help;
                 return 0;;
+            -n|--no-cd)
+                cd_after_activate=0;;
+            -c|--cd)
+                cd_after_activate=1;;
+            *)
+                if [ ${#out_args} -gt 0 ]
+                then
+                    out_args=( "${out_args[@]-}" "$a" )
+                else
+                    out_args=( "$a" )
+                fi;;
         esac
         i=$(( $i + 1 ))
     done
 
+    set -- "${out_args[@]}"
+
     typeset env_name="$1"
     if [ "$env_name" = "" ]
     then
         lsvirtualenv -b
         return 1
+    elif [ "$env_name" = "." ]
+    then
+        # The IFS default of breaking on whitespace causes issues if there
+        # are spaces in the env_name, so change it.
+        IFS='%'
+        env_name="$(basename $(pwd))"
+        unset IFS
     fi
 
     virtualenvwrapper_verify_workon_home || return 1
-    virtualenvwrapper_verify_workon_environment $env_name || return 1
+    virtualenvwrapper_verify_workon_environment "$env_name" || return 1
 
     activate="$WORKON_HOME/$env_name/$VIRTUALENVWRAPPER_ENV_BIN_DIR/activate"
     if [ ! -f "$activate" ]
@@ -746,7 +792,7 @@ function workon {
 
     }'
 
-    virtualenvwrapper_run_hook "post_activate"
+    VIRTUALENVWRAPPER_PROJECT_CD=$cd_after_activate virtualenvwrapper_run_hook "post_activate"
 
     return 0
 }
@@ -848,7 +894,7 @@ function cdsitepackages {
     virtualenvwrapper_verify_workon_home || return 1
     virtualenvwrapper_verify_active_environment || return 1
     typeset site_packages="`virtualenvwrapper_get_site_packages_dir`"
-    virtualenvwrapper_cd "$site_packages"/$1
+    virtualenvwrapper_cd "$site_packages/$1"
 }
 
 # Does a ``cd`` to the root of the currently-active virtualenv.
@@ -856,7 +902,7 @@ function cdsitepackages {
 function cdvirtualenv {
     virtualenvwrapper_verify_workon_home || return 1
     virtualenvwrapper_verify_active_environment || return 1
-    virtualenvwrapper_cd $VIRTUAL_ENV/$1
+    virtualenvwrapper_cd "$VIRTUAL_ENV/$1"
 }
 
 # Shows the content of the site-packages directory of the currently-active
@@ -908,9 +954,9 @@ function cpvirtualenv {
         echo "Please provide a valid virtualenv to copy."
         return 1
     else
-        # see if its already in workon
+        # see if it's already in workon
         if [ ! -e "$WORKON_HOME/$src_name" ]; then
-            # so its a virtualenv we are importing
+            # so it's a virtualenv we are importing
             # make sure we have a full path
             # and get the name
             src="$(virtualenvwrapper_expandpath "$src_name")"
@@ -1158,6 +1204,40 @@ function cdproject {
 function mktmpenv {
     typeset tmpenvname
     typeset RC
+    typeset -a in_args
+    typeset -a out_args
+
+    in_args=( "$@" )
+
+    if [ -n "$ZSH_VERSION" ]
+    then
+        i=1
+        tst="-le"
+    else
+        i=0
+        tst="-lt"
+    fi
+    typeset cd_after_activate=$VIRTUALENVWRAPPER_WORKON_CD
+    while [ $i $tst $# ]
+    do
+        a="${in_args[$i]}"
+        case "$a" in
+            -n|--no-cd)
+                cd_after_activate=0;;
+            -c|--cd)
+                cd_after_activate=1;;
+            *)
+                if [ ${#out_args} -gt 0 ]
+                then
+                    out_args=( "${out_args[@]-}" "$a" )
+                else
+                    out_args=( "$a" )
+                fi;;
+        esac
+        i=$(( $i + 1 ))
+    done
+
+    set -- "${out_args[@]}"
 
     # Generate a unique temporary name
     tmpenvname=$("$VIRTUALENVWRAPPER_PYTHON" -c 'import uuid,sys; sys.stdout.write(uuid.uuid4()+"\n")' 2>/dev/null)
@@ -1177,7 +1257,7 @@ function mktmpenv {
     fi
 
     # Change working directory
-    cdvirtualenv
+    [ "$cd_after_activate" = "1" ] && cdvirtualenv
 
     # Create the tmpenv marker file
     echo "This is a temporary environment. It will be deleted when you run 'deactivate'." | tee "$VIRTUAL_ENV/README.tmpenv"
@@ -1207,7 +1287,7 @@ function wipeenv {
         echo "Uninstalling packages:"
         cat "$req_file"
         echo
-        pip uninstall -y $(cat "$req_file" | sed 's/>/=/g' | cut -f1 -d=)
+        pip uninstall -y $(cat "$req_file" | grep -v '^-f' | sed 's/>/=/g' | cut -f1 -d=)
     else
         echo "Nothing to remove."
     fi
@@ -1222,6 +1302,9 @@ function allvirtualenv {
     virtualenvwrapper_verify_workon_home || return 1
     typeset d
 
+    # The IFS default of breaking on whitespace causes issues if there
+    # are spaces in the env_name, so change it.
+    IFS='%'
     virtualenvwrapper_show_workon_options | while read d
     do
         [ ! -d "$WORKON_HOME/$d" ] && continue
@@ -1234,6 +1317,7 @@ function allvirtualenv {
             "$@")
         echo
     done
+    unset IFS
 }
 
 #:help:virtualenvwrapper: show this help message
